/*****
*
*   Bezier_Curve.js
*   copyright 2001, Kevin Lindsey
*
*****/

function Bezier_Curve(x1,y1, x2,y2, x3,y3, x4,y4) {
    this.x1     = x1;
    this.y1     = y1;
    this.x2     = x2;
    this.y2     = y2;
    this.x3     = x3;
    this.y3     = y3;
    this.x4     = x4;
    this.y4     = y4;

    this.x      = 0;
    this.y      = 0;
    this.angle  = 0;
    
    this.shape    = null;
    this.parent   = null;
}


/*****
*
*    position_at
*
*****/
Bezier_Curve.prototype.position_at = function(percent) {
    var x5  = this.x1 + (this.x2 - this.x1) * percent;
    var y5  = this.y1 + (this.y2 - this.y1) * percent;
    var x6  = this.x2 + (this.x3 - this.x2) * percent;
    var y6  = this.y2 + (this.y3 - this.y2) * percent;
    var x7  = this.x3 + (this.x4 - this.x3) * percent;
    var y7  = this.y3 + (this.y4 - this.y3) * percent;
    var x8  = x5 + (x6 - x5) * percent;
    var y8  = y5 + (y6 - y5) * percent;
    var x9  = x6 + (x7 - x6) * percent;
    var y9  = y6 + (y7 - y6) * percent;
    var x10 = x8 + (x9 - x8) * percent;
    var y10 = y8 + (y9 - y8) * percent;

    this.angle = Math.PI/2 - Math.atan2(x9-x8, y9-y8);
    this.x = x10;
    this.y = y10;
};


/*****
*
*    show
*
*****/
Bezier_Curve.prototype.show = function(parent) {
    if (this.shape) {
        this.shape.node.setAttributeNS(null, "visibility", "visible");
    } else {
        this.parent = parent;
        
        this.shape = new Node_Builder(
            "path",
            { d: this.d_string(), stroke: "gray", fill: "none" }
        );
        this.shape.appendTo(parent);
    }
};


/*****
*
*    hide
*
*****/
Bezier_Curve.prototype.hide = function() {
    if (this.shape) {
        this.shape.node.setAttributeNS(null, "visibility", "hidden");
    }
};


/*****
*
*    toString
*
*****/
Bezier_Curve.prototype.toString = function() {
    return this.x + "," + this.y;
};


/*****
*
*    d_String
*
*****/
Bezier_Curve.prototype.d_string = function() {
    var text = "";
    
    text += "M" + this.x1 + "," + this.y1 + " ";
    text += "C" + this.x2 + "," + this.y2 + " ";
    text += this.x3 + "," + this.y3 + " ";
    text += this.x4 + "," + this.y4;
    
    return text;
};
