/*****
*
*	Color.js
*	copyright 2000, Kevin Lindsey
*
*****/

/*****
*
*	constructor
*
*****/
function Color(r, g, b) {
	if (arguments.length == 3) {
		this.r = r;
		this.g = g;
		this.b = b;
		this.h = 0;
		this.s = 0;
		this.v = 0;
		this.rgb2hsv();
	} else {
		this.r = 0;
		this.g = 0;
		this.b = 0;
		this.h = null;
		this.s = 0;
		this.v = 0
	}
}


/*****
*
*	set_r
*
*****/
Color.prototype.set_r = function(r) {
	this.r = r;
	this.rgb2hsv();
};


/*****
*
*	set_g
*
*****/
Color.prototype.set_g = function(g) {
	this.g = g;
	this.rgb2hsv();
};


/*****
*
*	set_b
*
*****/
Color.prototype.set_b = function(b) {
	this.b = b;
	this.rgb2hsv();
};


/*****
*
*	set_rgb
*
*****/
Color.prototype.set_rgb = function(r, g, b) {
	this.r = r;
	this.g = g;
	this.b = b;
	this.rgb2hsv();
};


/*****
*
*	set_h
*
*****/
Color.prototype.set_h = function(h) {
	this.h = h;
	this.hsv2rgb();
};


/*****
*
*	set_s
*
*****/
Color.prototype.set_s = function(s) {
	this.s = s;
	this.hsv2rgb();
};


/*****
*
*	set_v
*
*****/
Color.prototype.set_v = function(v) {
	this.v = v;
	this.hsv2rgb();
};


/*****
*
*	set_hsv
*
*****/
Color.prototype.set_hsv = function(h, s, v) {
	this.h = h;
	this.s = s;
	this.v = v;
	this.hsv2rgb();
};


/*****
*
*	rgb2hsv
*
*****/
Color.prototype.rgb2hsv = function() {
	var max, min;
	var delta;

	max   = (this.r > this.g) ? this.r : this.g;
	max   = (max    > this.b) ? max    : this.b;
	min   = (this.r < this.g) ? this.r : this.g;
	min   = (min    < this.b) ? min    : this.b;
	delta = max - min;

	this.v = max;
	if (max != 0) {
		this.s = Math.round(delta / max * 255);
	} else {
		this.s = 0;
	}

	if (this.s == 0) {
		this.h = null;
	} else {
		if (this.r == max) {
			this.h = (this.g - this.b) / delta;
		} else if (this.g == max) {
			this.h = 2 + (this.b - this.r) / delta;
		} else if (this.b == max) {
			this.h = 4 + (this.r - this.g) / delta;
		}
		this.h *= 60;
		if (this.h < 0) this.h += 360;
		this.h = Math.round(this.h);
	}
};


/*****
*
*	hsv2rgb
*
*****/
Color.prototype.hsv2rgb = function() {
	if (this.s == 0) {
		this.r = this.g = this.b = this.v;
	} else {
		var th = this.h;
		var ts = this.s / 255;
		if (th == 360) th = 0;
		th /= 60;
		var i = Math.floor(th);
		var f = th - i;
		var p = Math.round(this.v * (1 - ts));
		var q = Math.round(this.v * (1 - (ts * f)));
		var t = Math.round(this.v * (1 - (ts * (1 - f))));
		switch(i) {
			case 0:
				this.r = this.v; this.g = t; this.b = p;
				break;
			case 1:
				this.r = q; this.g = this.v; this.b = p;
				break;
			case 2:
				this.r = p; this.g = this.v; this.b = t;
				break;
			case 3:
				this.r = p; this.g = q; this.b = this.v;
				break;
			case 4:
				this.r = t; this.g = p; this.b = this.v;
				break;
			case 5:
				this.r = this.v; this.g = p; this.b = q;
				break;
		}
	}
};


/*****
*
*	equals
*
*****/
Color.prototype.equals = function(color) {
	this.r = color.r;
	this.g = color.g;
	this.b = color.b;
	this.h = color.h;
	this.s = color.s;
	this.v = color.v;
};


/*****
*
*	toString
*
*****/
Color.prototype.toString = function() {
	return "rgb(" + this.r + "," + this.g + "," + this.b + ")";
};
