/*****
*
*	Munition_Handler.js
*	Copyright 2000, Kevin Lindsey
*
*	assumes KEL_Game_Lib.js is loaded
*
*****/

function Munition_Handler(parent, shot_color, explosion_color) {
	this.parent     = parent;
	this.shots      = new Array();
	this.explosions = new Array();
	this.shot_color = shot_color;
	this.explosion_color = explosion_color;
	this.enemy = null;
}


/*****
*
*	new_shot
*
*****/
Munition_Handler.prototype.new_shot = function(x1, y1, x2, y2, speed) {
	var angle = Math.PI/2 - Math.atan2( x2-x1, y2-y1 );
	var dist  = Math.sqrt( (x2-x1)*(x2-x1) + (y2-y1)*(y2-y1) );
	var steps = dist / speed;
	var x_speed = (x2 - x1) / steps;
	var y_speed = (y2 - y1) / steps;


	angle *= 180 / Math.PI;
	while (angle < 0) angle += 360;

	var shot = new Projectile(
		x1, y1, 0, 0,
		this.parent,
		[
			new Node_Builder(
				"path",
				{ d: "M0,-3 A3,3 90 0,1 0 3 h -6 v -6 z", fill: this.shot_color }
			)
		]
	);
	shot.angle = angle;
	shot.step = 0;
	shot.steps = steps;
	shot.x_velocity = x_speed;
	shot.y_velocity = y_speed;
	this.shots[this.shots.length] = shot;
};


/*****
*
*	animate
*
*****/
Munition_Handler.prototype.animate = function() {
	this.animate_shots();
	this.animate_explosions();
};


/*****
*
*	animate_shots
*
*****/
Munition_Handler.prototype.animate_shots = function() {
	for (var i = 0; i < this.shots.length; i++) {
		var shot = this.shots[i];
		shot.step++;
		if (shot.step >= shot.steps) {
			shot.remove();
			for (var j = i; j < this.shots.length - 1; j++) {
				this.shots[j] = this.shots[j+1];
			}
			this.shots.length--;
			this.explode(shot.x, shot.y);
		} else {
			shot.advance();
		}
	}
};


/*****
*
*	explode
*
*****/
Munition_Handler.prototype.explode = function(x, y) {
	var explosion = new Projectile(
		x, y, 0, 0,
		this.parent,
		[
			new Node_Builder(
				"circle",
				{ cx: 0, cy: 0, r: 10, fill: this.explosion_color, opacity: "0.75" }
			)
		]
	);

	explosion.diff = 1;
	explosion.radius = 10;
	explosion.node = explosion.shapes[0].node;
	this.explosions[this.explosions.length] = explosion;
};


/*****
*
*	animate_explosions
*
*****/
Munition_Handler.prototype.animate_explosions = function() {
	for (var i = 0; i < this.explosions.length; i++) {
		var e = this.explosions[i];
		e.radius += e.diff;
		if (e.radius < 1) {
			e.remove();
			for (var j = i; j < this.explosions.length - 1; j++) {
				this.explosions[j] = this.explosions[j+1];
			}
			this.explosions.length--;
		} else {
			if (e.radius > 20) {
				e.diff = -e.diff;
				e.radius = 20;
				e.node.setAttributeNS(null, "r", e.radius);
			} else {
				e.node.setAttributeNS(null, "r", e.radius);
			}
			this.check_collision(e.x, e.y, e.radius);
		}
	}
};


/*****
*
*	check_collision
*
*****/
Munition_Handler.prototype.check_collision = function(x, y, r) {
	if (this.enemy) {
		for (var i = 0; i < this.enemy.shots.length; i++) {
			var shot = this.enemy.shots[i];
			var x_diff = shot.x - x;
			var y_diff = shot.y - y;
			var dist = Math.sqrt(x_diff*x_diff + y_diff*y_diff);
			if (dist <= r) {
				this.explode(shot.x, shot.y);
				shot.remove();
				for (var j = i; j < this.enemy.shots.length - 1; j++) {
					this.enemy.shots[j] = this.enemy.shots[j+1];
				}
				this.enemy.shots.length--;
				break;
			}
		}
	}
};


/*****
*
*	animation_pending
*
*****/
Munition_Handler.prototype.animation_pending = function() {
	var count;
	
	count  = this.shots.length;
	count += this.explosions.length;
	
	return (count > 0);
};
